library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity DailyStepper is
    port (
        Clk         : in std_logic;
        Reset       : in std_logic;
		FastForward : in std_logic;
		FastRewind  : in std_logic;
        StepperOut1 : out std_logic;
        StepperOut2 : out std_logic;
        StepperOut3 : out std_logic;
        StepperOut4 : out std_logic
    );
end entity;

architecture A of DailyStepper is
	constant COUNTER_RESET_VALUE      : integer := 2119725221;       -- ((24 * 60 * 60) segundos_por_vuelta / ((4076 / 2) pasos_por_vuelta) * 50000000 = 2119725221 ciclos por paso
	constant COUNTER_BITS             : integer := 32;               -- ceil(log2(2119725221)) = 31 bits para el contador
	constant COUNTER_LOAD_UPPER_BOUND : integer := 2119725000;       -- 2119725000
	constant COUNTER_LOAD_LOWER_BOUND : integer := 2069725000;       -- 2119725000 - 50000000 = 2069725000    (un segundo de pulso)
	constant SHIFT_BITS               : integer := 4;                -- motor de 4 fases
	constant COUNTER_QUICK_INCREMENT  : integer := 6000;
	signal ShiftDBus : std_logic_vector((SHIFT_BITS - 1) downto 0);
	signal ShiftQBus : std_logic_vector((SHIFT_BITS - 1) downto 0);
	signal ShiftMuxSel : std_logic_vector(1 downto 0);
	signal CounterDBus : std_logic_vector((COUNTER_BITS - 1) downto 0);
	signal CounterQBus : std_logic_vector((COUNTER_BITS - 1) downto 0);
	signal CounterMuxSel : std_logic;
	signal CounterIsZero : std_logic;
	signal CounterIsInLoadInterval : std_logic;
	signal CounterIncMuxOut : std_logic_vector((COUNTER_BITS - 1) downto 0);
	signal IsFast : std_logic;
	signal LatchDBus : std_logic_vector((SHIFT_BITS - 1) downto 0);
	signal LatchQBus : std_logic_vector((SHIFT_BITS - 1) downto 0);
	signal LatchMuxSel : std_logic;
	signal DirectionMuxOut : std_logic_vector((SHIFT_BITS - 1) downto 0);
	signal DirectionMuxSel : std_logic;
begin
	-- registro de desplazamiento
	process (Clk)
	begin
		if (Clk'event and (Clk = '1')) then
			ShiftQBus <= ShiftDBus;
		end if;
	end process;

	ShiftDBus <= std_logic_vector(to_signed(1, SHIFT_BITS)) when (ShiftMuxSel = "01") else
	             DirectionMuxOut when (ShiftMuxSel = "10") else
	             ShiftQBus;
	DirectionMuxOut <= (ShiftQBus(0) & ShiftQBus((SHIFT_BITS - 1) downto 1)) when (DirectionMuxSel = '0') else
	                   (ShiftQBus((SHIFT_BITS - 2) downto 0) & ShiftQBus(SHIFT_BITS - 1));

	-- latch de salida
	process (Clk)
	begin
		if (Clk'event and (Clk = '1')) then
			LatchQBus <= LatchDBus;
		end if;
	end process;

	LatchDBus <= ShiftQBus when ((LatchMuxSel = '1') or (IsFast = '1')) else
	             std_logic_vector(to_signed(0, SHIFT_BITS));

	-- contador
	process (Clk)
	begin
		if (Clk'event and (Clk = '1')) then
			CounterQBus <= CounterDBus;
		end if;
	end process;

	CounterIncMuxOut <= std_logic_vector(to_signed(-COUNTER_QUICK_INCREMENT, COUNTER_BITS)) when (IsFast = '1') else
	                    std_logic_vector(to_signed(-1, COUNTER_BITS));
	CounterDBus <= std_logic_vector(to_signed(COUNTER_RESET_VALUE, COUNTER_BITS)) when (CounterMuxSel = '1') else
	               std_logic_vector(to_signed(to_integer(signed(CounterQBus)) + to_integer(signed(CounterIncMuxOut)), COUNTER_BITS));
	CounterIsZero <= '1' when (to_integer(signed(CounterQBus)) <= 0) else
	                 '0';
	CounterIsInLoadInterval <= '1' when ((to_integer(signed(CounterQBus)) <= COUNTER_LOAD_UPPER_BOUND) and (to_integer(signed(CounterQBus)) >= COUNTER_LOAD_LOWER_BOUND)) else
	                           '0';

	-- lógica de control (basada en el contador, no en máquina de estados)
	CounterMuxSel <= '1' when ((Reset = '0') or ((Reset = '1') and (CounterIsZero = '1'))) else
	                 '0';
	ShiftMuxSel <= "01" when (Reset = '0') else
	               "10" when ((Reset = '1') and (CounterIsZero = '1')) else
	               "00";
	LatchMuxSel <= '1' when (CounterIsInLoadInterval = '1') else
	               '0';

	-- lógica de control de la velocidad (entradas FastForward y FastRewind)
	IsFast <= '1' when ((FastForward = '1') or (FastRewind = '1')) else
	          '0';
	DirectionMuxSel <= '1' when ((FastForward = '0') and (FastRewind = '1')) else
	                   '0';

	-- salida
	StepperOut1 <= LatchQBus(0);
	StepperOut2 <= LatchQBus(1);
	StepperOut3 <= LatchQBus(2);
	StepperOut4 <= LatchQBus(3);
end architecture;
